/*
    SDL_net:  An example cross-platform network library for use with SDL
    Copyright (C) 1997-2004 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.net;

import org.libsdl.endian;
import org.libsdl.error;
import org.libsdl.stdinc;
import org.libsdl.versions;

immutable uint SDL_NET_MAJOR_VERSION = 1;
immutable uint SDL_NET_MINOR_VERSION = 2;
immutable uint SDL_NET_PATCHLEVEL    = 7;

void SDL_NET_VERSION(ref SDL_version ver)
{
	ver.major = SDL_NET_MAJOR_VERSION;
	ver.minor = SDL_NET_MINOR_VERSION;
	ver.patch = SDL_NET_PATCHLEVEL;
}

extern (C)
{
	const(SDL_version)* SDLNet_Linked_Version();
	int SDLNet_Init();
	void SDLNet_Quit();
}

struct IPaddress
{
	Uint32 host;
	Uint16 port;
}

immutable Uint32 INADDR_ANY       = 0x00000000;
immutable Uint32 INADDR_NONE      = 0xFFFFFFFF;
immutable Uint32 INADDR_BROADCAST = 0xFFFFFFFF;

extern (C)
{
	int SDLNet_ResolveHost(IPaddress* address, const(char)* host, Uint16 port);
	const(char)* SDLNet_ResolveIP(IPaddress* ip);
}

struct _TCPsocket;
alias _TCPsocket* TCPsocket;

extern (C)
{
	TCPsocket SDLNet_TCP_Open(IPaddress* ip);
	TCPsocket SDLNet_TCP_Accept(TCPsocket server);
	IPaddress* SDLNet_TCP_GetPeerAddress(TCPsocket sock);
	int SDLNet_TCP_Send(TCPsocket sock, const(void)* data, int len);
	int SDLNet_TCP_Recv(TCPsocket sock, void* data, int maxlen);
	void SDLNet_TCP_Close(TCPsocket sock);
}

immutable int SDLNET_MAX_UDPCHANNELS  = 32;
immutable int SDLNET_MAX_UDPADDRESSES = 4;

struct _UDPsocket;
alias _UDPsocket* UDPsocket;

struct UDPpacket
{
	int channel;
	Uint8* data;
	int len;
	int maxlen;
	int status;
	IPaddress address;
}

extern (C)
{
	UDPpacket* SDLNet_AllocPacket(int size);
	int SDLNet_ResizePacket(UDPpacket* packet, int newsize);
	void SDLNet_FreePacket(UDPpacket* packet);
	UDPpacket** SDLNet_AllocPacketV(int howmany, int size);
	void SDLNet_FreePacketV(UDPpacket** packetV);
	UDPsocket SDLNet_UDP_Open(Uint16 port);
	int SDLNet_UDP_Bind(UDPsocket sock, int channel, IPaddress* address);
	void SDLNet_UDP_Unbind(UDPsocket sock, int channel);
	IPaddress* SDLNet_UDP_GetPeerAddress(UDPsocket sock, int channel);
	int SDLNet_UDP_SendV(UDPsocket sock, UDPpacket** packets, int npackets);
	int SDLNet_UDP_Send(UDPsocket sock, int channel, UDPpacket* packet);
	int SDLNet_UDP_RecvV(UDPsocket sock, UDPpacket** packets);
	int SDLNet_UDP_Recv(UDPsocket sock, UDPpacket* packet);
	void SDLNet_UDP_Close(UDPsocket sock);
}

struct _SDLNet_SocketSet;
alias _SDLNet_SocketSet* SDLNet_SocketSet;

struct _SDLNet_GenericSocket
{
	int ready;
}
alias _SDLNet_GenericSocket* SDLNet_GenericSocket;

extern (C)
{
	SDLNet_SocketSet SDLNet_AllocSocketSet(int maxsockets);
	int SDLNet_AddSocket(SDLNet_SocketSet set, SDLNet_GenericSocket sock);
	int SDLNet_DelSocket(SDLNet_SocketSet set, SDLNet_GenericSocket sock);
}

int SDLNet_TCP_AddSocket(SDLNet_SocketSet set, TCPsocket sock)
{
	return SDLNet_AddSocket(set, cast(SDLNet_GenericSocket)sock);
}

int SDLNet_UDP_AddSocket(SDLNet_SocketSet set, UDPsocket sock)
{
	return SDLNet_AddSocket(set, cast(SDLNet_GenericSocket)sock);
}

int SDLNet_TCP_DelSocket(SDLNet_SocketSet set, TCPsocket sock)
{
	return SDLNet_DelSocket(set, cast(SDLNet_GenericSocket)sock);
}

int SDLNet_UDP_DelSocket(SDLNet_SocketSet set, UDPsocket sock)
{
	return SDLNet_DelSocket(set, cast(SDLNet_GenericSocket)sock);
}

extern (C)
{
	int SDLNet_CheckSockets(SDLNet_SocketSet set, Uint32 timeout);
}

bool SDLNet_SocketReady(TCPsocket sock)
{
	return sock !is null && (cast(SDLNet_GenericSocket)sock).ready != 0;
}

bool SDLNet_SocketReady(UDPsocket sock)
{
	return sock !is null && (cast(SDLNet_GenericSocket)sock).ready != 0;
}

extern (C)
{
	void SDLNet_FreeSocketSet(SDLNet_SocketSet set);
	void SDLNet_Write16(Uint16 value, void* area);
	void SDLNet_Write32(Uint32 value, void* area);
	Uint16 SDLNet_Read16(void* area);
	Uint32 SDLNet_Read32(void* area);
}

alias SDL_SetError SDLNet_SetError;
alias SDL_GetError SDLNet_GetError;

void SDLNet_Write16(Uint16 value, void* areap)
{
	*(cast(Uint16*)areap) = SDL_SwapBE16(value);
}

void SDLNet_Write32(Uint32 value, void* areap)
{
	*(cast(Uint32*)areap) = SDL_SwapBE32(value);
}

Uint16 SDLNet_Read16(void* areap)
{
	return SDL_SwapBE16(*(cast(Uint16*)areap));
}

Uint32 SDLNet_Read32(void* areap)
{
	return SDL_SwapBE32(*(cast(Uint32*)areap));
}
